/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

/*
 * Methods to work with messages.
 */
define([
    "aps/load", "dijit/registry", "dojo/json", "aps/Message"
],
function (load, registry, json, Message) {
    var pageId = "apsPageContainer";

    function removeAll() {
        var messageList = registry.byId(pageId) && registry.byId(pageId).get("messageList");
        messageList && messageList.removeAll();
    }

    function parseError(error) {
        var errorData,
            message = error.message;

        try {
            var text = error.response.text.replace(/(?! )\s/g, '');
            errorData = json.parse(text);
            if (errorData.message) {
                message = errorData.message.replace("Internal Server Error:", "");
            }
        } catch (e) {
        }

        return message;
    }

    function addMessage(text, type) {
        registry.byId(pageId).get("messageList").addChild(new Message({ description: text, type: type }));
    }

    function displayMessage(description, type) {
        var page = registry.byId(pageId);
        if (page) {
            addMessage(description, type);
            return;
        }
        load([
            ["aps/PageContainer", { id: pageId }, []]
        ]).then(function () {
            addMessage(description, type);
        });
    }

    return {
        parseError: parseError,
        
        displayError: function (error) {
            aps.apsc.cancelProcessing();
            aps.apsc.hideLoading();

            removeAll();
            displayMessage(parseError(error), "error");
        },

        displayErrorAndGo: function (error) {
            displayMessage(parseError(error), "error");
        },

        displayWarning: function (warning, preserveOthers) {
            if (!preserveOthers) {
                removeAll();
            }
            displayMessage(warning, "warning");
        },

        displayMessage: displayMessage,
        removeAll : removeAll
    };
});
