/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
    "aps/nav/ViewPlugin",
    "aps/Status",
    "aps/Tile",
    "aps/UsageInfo",
    "aps/xhr",
    "dojo/_base/declare",
    "dojo/Deferred",
    "dojo/promise/all",
    "./../../common",
    "./../../icons",
    "./../../messages",
    "./../../uomConverter"
], function (
    ViewPlugin,
    Status,
    Tile,
    UsageInfo,
    xhr,
    declare,
    Deferred,
    all,
    common,
    icons,
    messages,
    uomConverter
) {
    var serviceStatuses = {
            ORDERED: 10,
            STOPPED: 30,
            RUNNING: 50,
            REMOVED: 90
        },
        subscriptionStatuses = {
            ORDERED: 10,
            TRIAL: 15,
            ACTIVE: 30,
            GRACED: 40,
            EXPIRED: 50,
            TERMINATED: 60,
            ADMINISTRATIVE_HOLD: 80,
            CREDIT_HOLD: 85,
            CREDIT_ADMINISTRATIVE_HOLD: 89
        };

    return declare(ViewPlugin, {
        _tile: null,
        _tenant: null,
        _billing: null,

        init: function (mediator) {
            var initResult = new Deferred();
            var self = this;
            self._tile = new Tile({
                id: common.genId("backup-dashboard-tile"),
                title: _("Cyber Protection", {}, {}, self),
                gridSize: 'md-4 xs-12',
                onClick: function() {
                    self.nav.gotoView(common.APP_ID + (self._tenant.type == common.TENANT_TYPE.PARTNER ? "#nextcp-ccp-partner" : "#nextcp-ccp"));
                },
                isBusy: true,
                fontColor: "#ffffff",
                backgroundColor: "#163584",
                iconName: self.buildStaticURL(icons.ACRONIS_BACKUP_TILE_150)
            });

            mediator.getWidget = function () {
                return self._tile;
            };

            mediator.watch("resourceUsage", function (name) {
                if (name !== "resourceUsage") {
                    return;
                }
                var currentSubscriptionDeferred = new Deferred();
                if (self._tenant && self._billing && self._billing.aps) {
                    xhr.get("/aps/2/resources/" + self._billing.aps.id + "/sm/subscriptions?subscriptionId=" + self._tenant.subscription.subscriptionId).then(function (billingSubscriptions) {
                        if (billingSubscriptions.length !== 1) {
                            currentSubscriptionDeferred.reject(_("The subscription #__id__ is not found.", {id: self._tenant.subscription.subscriptionId}, {}, self));
                        }
                        var currentSubscription = billingSubscriptions[0];
                        currentSubscriptionDeferred.resolve(currentSubscription);
                    });

                } else {
                    currentSubscriptionDeferred.resolve();
                }

                currentSubscriptionDeferred.then(function (currentSubscription) {
                    var activeSubscriptionStatuses = [subscriptionStatuses.TRIAL, subscriptionStatuses.ACTIVE, subscriptionStatuses.GRACED];
                    // there is an active subscription
                    if (currentSubscription && activeSubscriptionStatuses.indexOf(currentSubscription.status) != -1) {
                        self._renderActiveTile(currentSubscription);
                    }
                    // no active subscription
                    else {
                        all({
                            trialAvailable: self._isTrialAvailable(),
                            provisioning: self._isProvisioning()
                        }).then(function (result) {
                            if (result.provisioning) {
                                self._renderInProvisioning();
                            } else {
                                self._renderNoSubscription();
                                if (result.trialAvailable) {
                                    self._renderTrialAvailable();
                                }
                            }
                        }, messages.displayError);
                    }
                }, messages.displayError);
            });
            initResult.resolve();
            return initResult;
        },
        _subscribe: function () {
            var self = this;
            self.nav.gotoView("http://www.parallels.com/ccp-billing-sales-wizard#wizard", null, {
                "appId": common.APP_ID,
                "title": _("Enable Backup", {}, {}, self),
                "withDomain": false
            });
        },
        onContext: function (context) {
            var self = this;
            self._tenant = context.vars.tenant;
            self._billing = context.vars.billing;
            if (!self._billing && (!self._tenant || self._tenant.subscription.disabled)) {
                self._tile.set('visible', false);
            }
        },
        _renderTrialAvailable: function () {
            var self = this;
            self._tile.addChild(new Status({
                id: common.genId("backup-dashboard-status"),
                status: "custom",
                useIcon: true,
                statusInfo: {
                    custom: {
                        "label": _("Free Trial Available!", {}, {}, self),
                        "type": "warning"
                    }
                }
            }));
        },
        _isTrialAvailable: function () {
            var self = this;
            return xhr("/aps/2/resources/" + self._billing.aps.id + "/bss/account/service-info?appId=" + common.APP_ID).then(function (serviceInfo) {
                return serviceInfo.hasTrial;
            });
        },
        _isProvisioning: function () {
            var self = this;
            return xhr('/aps/2/resources/' + self._billing.aps.id + '/sm/subscriptions-by-app-id?appId=' + common.APP_ID).then(function (allSubscriptions) {
                var inProvisioning = allSubscriptions.filter(function (subscr) {
                    return subscr.status === subscriptionStatuses.ORDERED;
                });
                return inProvisioning.length > 0;
            });
        },
        _renderNoSubscription: function () {
            var self = this;
            self._tile.removeAll();
            self._tile.set({
                isBusy: false,
                info: null,
                onClick: function () {
                    self._subscribe();
                }
            });
            self._tile.addChild(new UsageInfo({
                textFirstNumber: "",
                description: _("Back up All Your Data", {}, {}, self),
                showPie: false
            }));
            self._tile.set("buttons", [{
                type: "default",
                label: _("Enable Backup", {}, {}, self),
                iconClass: "fa-plus",
                onClick: function () {
                    self._subscribe();
                },
                autoBusy: false
            }
            ]);
        },
        _renderInProvisioning: function () {
            var self = this;
            self._tile.removeAll();
            self._tile.set({
                isBusy: true,
                buttons: [],
                onClick: null
            });
            self._tile.addChild(new UsageInfo({
                textFirstNumber: "",
                description: _("Please Wait...", {}, {}, self),
                hint: _("The service is being provisioned", {}, {}, self),
                showPie: false
            }));
        },
        _getTrialLeftBtn: function (billingSubscription) {
            var self = this;
            return {
                type: "default",
                link: true,
                label: _("Upgrade to Paid", {}, {}, self),
                iconClass: "fa-shopping-cart",
                subscriptionId: billingSubscription.subscriptionId,
                onClick: function () {
                    this.cancel && this.cancel();
                    self.nav.gotoView("http://www.parallels.com/ccp-billing-subscription-management#upgrade-wizard", null, {
                        subscriptionId: this.subscriptionId,
                        appIds: [common.APP_ID]
                    });
                },
                visible: true
            };
        },
        _getTrialDaysLeft: function (billingSubscription) {
            var self = this,
                trialDaysLeft = "";
            if ("expirationDate" in billingSubscription)
                trialDaysLeft = Math.ceil((Date.parse(billingSubscription.expirationDate) - Date.now()) / 1000 / (60 * 60 * 24));
            else
                trialDaysLeft = billingSubscription.period;

            return trialDaysLeft == 1 ? _("1 day left", {}, {}, self) : _("__days__ days left", {
                days: trialDaysLeft
            }, {}, self);
        },
        _renderActiveTile: function (billingSubscription) {
            var self = this;
            all({
                statistics: xhr.get("/aps/2/resources/" + self._tenant.aps.id + "/updateStatistics"),
                allCounters: xhr.get("/aps/2/resources/" + self._tenant.aps.id + "/allCounters")
            }).then(function (response) {
                window.allCounters = response.allCounters.counters;
                var textSuffix, usage, limit,
                    tenant = response.statistics.tenant,
                    allowedCounters = common.getTenantVisibleQuotas(tenant, "storage_usage_only", false),
                    storageCounter = Object.values(allowedCounters)[0] || "StorageUsage",
                    isUnlimited = tenant[storageCounter].limit == common.QUOTA_UNLIMITED_VALUE;

                if (isUnlimited) {
                    var usageSize = uomConverter.convertByteToMajorityUnit(tenant[storageCounter].usage);
                    usage = usageSize.value;
                    limit = "";
                    textSuffix = uomConverter.getUnitName(usageSize.unit, {}, {}, self);
                }
                else {
                    var limitSize = uomConverter.convertByteToMajorityUnit(tenant[storageCounter].limit);
                    usage = uomConverter.convert(tenant[storageCounter].usage, 'b', limitSize.unit);
                    usage = Number(usage.toFixed(2));
                    limit = limitSize.value;
                    textSuffix = uomConverter.getUnitName(limitSize.unit, {}, {}, self);
                }

                self._tile.removeAll();
                self._tile.set("isBusy", false);
                self._tile.set("onClick", function() {
                    self.nav.gotoView(common.APP_ID + (self._tenant.type == common.TENANT_TYPE.PARTNER ? "#nextcp-ccp-partner" : "#nextcp-ccp"));
                });

                self._tile.addChild(new UsageInfo({
                    value: usage,
                    maximum: limit,
                    textFirstNumber: String(usage),
                    textSecondNumber: String(limit),
                    textSuffix: textSuffix,
                    description: _("Cloud Storage Used", {}, {}, self),
                    showPie: false, //!isUnlimited,
                    onClick: null
                }));
                if (billingSubscription.trial && billingSubscription.serviceStatus === serviceStatuses.RUNNING) {
                    self._tile.set("buttons", [self._getTrialLeftBtn(billingSubscription)]);
                    self._tile.set('info', new Status({
                        status: 'trialDaysLeft',
                        useIcon: true,
                        statusInfo: {
                            trialDaysLeft: {
                                'label': self._getTrialDaysLeft(billingSubscription),
                                'type': 'warning'
                            }
                        }
                    }));
                } else {
                    self._tile.set({
                        buttons: [],
                        info: null
                    });
                }
            }, messages.displayError);
        }
    });
});
