/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

require([
    "aps/load",
    "aps/Memory",
    "app/common",
    "app/messages",
    "app/storagesHelper",
    "app/uomConverter",
    "aps/xhr",
    "dojo/Deferred",
    "dojo/promise/all"
], function (
    load,
    Memory,
    common,
    messages,
    storagesHelper,
    uomConverter,
    xhr,
    Deferred,
    all
) {
    var globals = aps.context.vars.globals;

    function loadStoragesForGrid() {
        var deferred = new Deferred(),
            gridData = [];
        xhr.get("/aps/2/resources/" + globals.aps.id + "/datacenters").then(function (datacenters) {
            var requests = {};
            datacenters.forEach(function (datacenter) {
                requests[datacenter.aps.id] = xhr.get("/aps/2/resources/" + datacenter.aps.id + "/storages");
            });
            all(requests).then(function (response) {
                datacenters.forEach(function (datacenter) {
                    var storages = response[datacenter.aps.id];
                    storages.forEach(function (storage) {
                        gridData.push({
                            datacenterId: datacenter.id,
                            datacenterName: common.escapeAngularBrackets(datacenter.name),
                            storageId: storage.id,
                            storageName: common.escapeAngularBrackets(storage.name),
                            storageInherited: storage.inherited,
                            storageUid: storage.uid,
                            storageSpaceUsage: storage.spaceUsage,
                            optionId: storage.optionId
                        });
                    });
                });
                deferred.resolve(gridData);
            }, deferred.reject);
        }, deferred.reject);
        return deferred;
    }

    loadStoragesForGrid().then(function (gridData) {

        load(["aps/PageContainer", {id: "apsPageContainer"}, [
            ["aps/Output", {
                id: common.genId('output'),
                content: _('A list of backup storage enabled for your accounts.')
            }],
            ["aps/Grid", {
                id: common.genId('grid'),
                showPaging: false,
                pageSizeOptions: "GRID_UNLIMITED",
                columns: [
                    {field: "optionId", name: _("Option ID")},
                    {field: "storageName", name: _("Name")},
                    {field: "datacenterName", name: _("Datacenter")},
                    {
                        field: "storageInherited", name: _("Inherited"),
                        renderCell: function (row) {
                            return row.storageInherited == common.STORAGE_INHERITED.YES ? _("Yes") : _("No");
                        }
                    },
                    {field: "storageUid", name: _("GUID")},
                    {
                        field: "storageSpaceUsage", name: _("Space Usage"),
                        renderCell: function (row) {
                            var usage = uomConverter.convertByteToMajorityUnit(row.storageSpaceUsage);
                            return usage.value + " " + _(usage.unit.toUpperCase());
                        }
                    }
                ],
                store: new Memory({data: gridData, idProperty: "storageUid"})
            }]
        ]]);
    });
});