/*
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

define([
    "dojo/_base/declare",
    "dojo/query",
    "aps/Password"
], function(declare, query, Password) {
    return declare(Password, {
        passwordVisibleHint: _("Hide password"),
        passwordHideHint: _("Show password"),
        resetMessage: _("Generate"),
        resetButtonHint: _("Generate password"),
        showMessage: _("Show"),
        hideMessage: _("Hide"),
        notMatchedMessage: _("Password does not match"),
        confirmation: true,
        showResetButton: true,
        showStrengthIndicator: true,
        showShowPasswordButton: true,
        isValid: function () {
            return this.strength >= 3;
        },
        /**
         * Generates password of specified length.
         * @param len Length of password to generate
         * @returns {string} Password generated
         */
        _generate: function(len) {
            // vars for password generator
            var digits = [0,1,2,3,4,5,6,7,8,9],
                chars = ['a','b','c','d','e','f','g','h','j','k','l','m','n','o','p','r','s','t','u','v','w','x','y','z'],
                spec = ['!','@','#','$','%','^','&','*','(',')','-','_','+','='];
            
            var getRandomItemFromArray = function (arr) {
                return arr[Math.floor(Math.random() * arr.length)];
            };

            var out = '';
            for (var i = 0; i < len; i++) {
                var ch = Math.random();
                if (ch < 0.25) {
                    out += getRandomItemFromArray(digits);
                } else if (ch < 0.5) {
                    out += getRandomItemFromArray(chars);
                } else if (ch < 0.75){
                    out += getRandomItemFromArray(spec);
                } else {
                    out += getRandomItemFromArray(chars).toUpperCase();
                }
            }
            return out;
        },
        _reset: function () {
            if (this.disabled) {
                return;
            }

            var password = this._generate(10);
            this.set("value", password);
            this.confirmationNode.set("value", password);
            this.set("passwordVisible", true);
        }
    });
});

