<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\CloudApi;

use Acronis\Cloud\Client\Api\AccessApi;
use Acronis\Cloud\Client\Api\ApplicationsApi;
use Acronis\Cloud\Client\Api\BillingApi;
use Acronis\Cloud\Client\Api\IdpApi;
use Acronis\Cloud\Client\Api\InfraApi;
use Acronis\Cloud\Client\Api\ReportsApi;
use Acronis\Cloud\Client\Api\TenantsApi;
use Acronis\Cloud\Client\Api\UsersApi;

abstract class AwareApi extends BaseApi
{
    /** @var IdpApi */
    private $idpApi;

    /** @var TenantsApi */
    private $tenantsApi;

    /** @var AccessApi */
    private $accessApi;

    /** @var InfraApi */
    private $infraApi;

    /** @var BillingApi */
    private $billingApi;

    /** @var ApplicationsApi */
    private $applicationsApi;

    /** @var UsersApi */
    private $usersApi;

    /** @var ReportsApi */
    private $reportsApi;

    /* ====== Getters for Swagger Client Api instances ====== */

    /**
     * @return IdpApi
     */
    protected function getIdpApi()
    {
        if (!$this->idpApi) {
            $this->idpApi = $this->createIdpApi();
        }
        return $this->idpApi;
    }

    /**
     * @return TenantsApi
     */
    protected function getTenantsApi()
    {
        if (!$this->tenantsApi) {
            $this->tenantsApi = $this->createTenantsApi();
        }
        return $this->tenantsApi;
    }

    /**
     * @return UsersApi
     */
    protected function getUsersApi()
    {
        if (!$this->usersApi) {
            $this->usersApi = $this->createUsersApi();
        }
        return $this->usersApi;
    }

    /**
     * @return ApplicationsApi
     */
    protected function getApplicationsApi()
    {
        if (!$this->applicationsApi) {
            $this->applicationsApi = $this->createApplicationsApi();
        }
        return $this->applicationsApi;
    }

    /**
     * @return BillingApi
     */
    protected function getBillingApi()
    {
        if (!$this->billingApi) {
            $this->billingApi = $this->createBillingApi();
        }
        return $this->billingApi;
    }

    /**
     * @return InfraApi
     */
    protected function getInfraApi()
    {
        if (!$this->infraApi) {
            $this->infraApi = $this->createInfraApi();
        }
        return $this->infraApi;
    }

    /**
     * @return AccessApi
     */
    protected function getAccessApi()
    {
        if (!$this->accessApi) {
            $this->accessApi = $this->createAccessApi();
        }
        return $this->accessApi;
    }

    /**
     * @return ReportsApi
     */
    protected function getReportsApi()
    {
        if (!$this->reportsApi) {
            $this->reportsApi = $this->createReportsApi();
        }
        return $this->reportsApi;
    }

    /* ====== Factory methods to create Swagger Client Api instances ====== */

    /**
     * @return IdpApi
     */
    private function createIdpApi()
    {
        return new IdpApi($this->getApiClient());
    }

    /**
     * @return TenantsApi
     */
    private function createTenantsApi()
    {
        return new TenantsApi($this->getApiClient());
    }

    /**
     * @return UsersApi
     */
    private function createUsersApi()
    {
        return new UsersApi($this->getApiClient());
    }

    /**
     * @return ApplicationsApi
     */
    private function createApplicationsApi()
    {
        return new ApplicationsApi($this->getApiClient());
    }

    /**
     * @return BillingApi
     */
    private function createBillingApi()
    {
        return new BillingApi($this->getApiClient());
    }

    /**
     * @return InfraApi
     */
    private function createInfraApi()
    {
        return new InfraApi($this->getApiClient());
    }

    /**
     * @return AccessApi
     */
    private function createAccessApi()
    {
        return new AccessApi($this->getApiClient());
    }

    /**
     * @return ReportsApi
     */
    private function createReportsApi()
    {
        return new ReportsApi($this->getApiClient());
    }
}