<?php
/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Console\Command\Repair;


use Acronis\App\Console\Command\ApsInstanceCommand;
use Acronis\App\Console\Command\LockableTrait;
use Acronis\App\Console\Command\Loggable;
use Acronis\App\Services\Apsc\ApscInstanceFactoryTrait;
use APS\Proto;
use Json;
use RqlBuilder;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Lock\Factory;
use Symfony\Component\Lock\Store\RedisStore;
use tenant;

class DatacenterLinks extends Loggable
{
    use LockableTrait;
    use ApscInstanceFactoryTrait;


    protected function getCommandName()
    {
        return 'repair:detacenter-links';
    }

    protected function getCommandDescription()
    {
        return 'Command for checking/fixing broken datacenter links';
    }

    protected function getCommandHelp()
    {
        return 'Command for checking/fixing broken datacenter links';
    }

    protected function configure()
    {
        parent::configure();

        $this->addArgument(
            'mode',
            InputArgument::REQUIRED,
            'Possible value check|fix'
        );
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $mode = $input->getArgument('mode');
        switch ($mode) {
            case 'check':
                foreach (\APS\ControllerProxy::listInstances() as $apsInstanceId) {
                    $apsc = $this->createApscInstance($apsInstanceId);
                    /** @var tenant $tenant */
                    foreach ($this->getTenants($apsc) as $tenant) {
                        $this->getLog()->notice("APS resource ID: '{$tenant->aps->id}'. APS status '{$tenant->aps->status}'). Datacenter ID' {$tenant->datacenterId}'. OSA subscription ID: '{$tenant->subscriptionId}'. OSA account ID: '{$tenant->account->id}'");
                    }
                }
                break;
            case 'fix':
                foreach (\APS\ControllerProxy::listInstances() as $apsInstanceId) {
                    $apsc = $this->createApscInstance($apsInstanceId);
                    $datacenters = $this->getDatacenters($apsc);
                    /** @var tenant $tenant */
                    foreach ($this->getTenants($apsc) as $tenant) {
                        $this->getLog()->notice("APS resource ID: '{$tenant->aps->id}'. APS status '{$tenant->aps->status}'). Datacenter ID' {$tenant->datacenterId}'. OSA subscription ID: '{$tenant->subscriptionId}'. OSA account ID: '{$tenant->account->id}'");
                        if ($tenant->aps->status !== 'aps:ready') {
                            $this->getLog()->warning("Skip because APS resource status is '{$tenant->aps->status}'");
                            continue;
                        }
                        $datacenterId = $tenant->datacenterId;
                        if (!array_key_exists($datacenterId, $datacenters)) {
                            $this->getLog()->warning("Datacenter with id '{$datacenterId}' not found");
                            continue;
                        }
                        $tenant->datacenter->set($datacenters[$datacenterId]);
                        $apsc->linkResource($tenant, tenant::LINK_NAME_DATACENTER, $tenant->datacenter);
                        $this->getLog()->notice('Link was fixed');
                    }
                }
                break;
            default:
                throw new \Exception("Unsupported mode '{$mode}'");
        }
    }

    private function getDatacenters($apsc)
    {
        $rqlFilter = RqlBuilder::createInstance()
            ->implementing(\APSResource::CLOUD_BACKUP_DATACENTER_TYPE)
            ->getQuery();
        $datacenters = [];
        foreach($apsc->getResources($rqlFilter) as $datacenter)
        {
            $datacenters[$datacenter->id] = $datacenter;
        }

        return $datacenters;
    }

    private function getTenants($apsc)
    {
        $rqlFilter = RqlBuilder::createInstance()
            ->implementing(\APSResource::CLOUD_BACKUP_TENANT_TYPE)
            ->select(['datacenter.aps.id', 'account', 'subscription'])
            ->eq('datacenter.aps.id', 'null()')
            ->getQuery();

        return $apsc->getResources($rqlFilter);
    }
}