<?php
/**
 * @Copyright © 2002-2019 Acronis International GmbH. All rights reserved
 */

class Model_ManagerFactory implements Model_ManagerFactoryInterface
{
    /**
     * @param Service_MspApiClient_ApiInterface $mspApi
     * @return Model_UserManagerInterface
     */
    public function getUserManager(Service_MspApiClient_ApiInterface $mspApi)
    {
        return new Model_UserManager($mspApi);
    }

    /**
     * @param \APS\ControllerProxy $apsc
     * @param \APS\ResourceProxy | \APS\ResourceBase $global
     * @param \APS\ResourceProxy | \APS\ResourceBase $datacenter
     * @param Service_MspApiClient_ApiInterface $mspApi
     * @param Service_OpenApi_ApiInterface $openApi
     * @return Model_MspGroup_OpenApiRetrieverInterface
     */
    public function getOpenApiRetrieverForDatacenter(
        $apsc,
        $global,
        $datacenter,
        Service_MspApiClient_ApiInterface $mspApi,
        Service_OpenApi_ApiInterface $openApi
    )
    {
        return new Model_MspGroup_OpenApiRetriever(
            new Model_ResellerMapByDatacenterHelper($apsc, $global, $datacenter), $mspApi, $openApi
        );
    }

    /**
     * @param Service_MspApiClient_ApiInterface $mspApi
     * @param Service_APSApi_ApiInterface $apsApi
     * @return Model_MspGroupManager
     */
    public function getMspGroupManager(Service_MspApiClient_ApiInterface $mspApi, Service_APSApi_ApiInterface $apsApi)
    {
        return new Model_MspGroupManager($mspApi, $apsApi);
    }

    /**
     * @param Service_MspApiClient_ApiInterface $mspApi
     * @return Model_TenantManager
     */
    public function getTenantManager(Service_MspApiClient_ApiInterface $mspApi)
    {
        return new Model_TenantManager($mspApi);
    }

    /**
     * @param users $backupAccount
     * @return Model_UserScreenNotificationManagerInterface
     */
    public function getUserScreenNotificationManager($backupAccount)
    {
        return new Model_UserScreenNotificationManager($backupAccount);
    }

    /**
     * @param Service_PbaApi_ApiInterface $pbaApi
     * @return Model_OSA_SubscriptionParametersManagerInterface
     */
    public function getSubscriptionParametersManager(Service_PbaApi_ApiInterface $pbaApi)
    {
        return new Model_OSA_SubscriptionParametersManager($pbaApi);
    }

    /**
     * @param Service_PbaApi_ApiInterface $pbaApi
     * @return Model_OSA_AccountAttributesManagerInterface
     */
    public function getAccountAttributesManager(Service_PbaApi_ApiInterface $pbaApi)
    {
        return new Model_OSA_AccountAttributesManager($pbaApi);
    }

    /**
     * @param Service_OpenApi_ApiInterface $openApi
     * @param Service_PbaApi_ApiInterface $pbaApi
     * @return Model_MinimalCommitmentDiscountManager
     */
    public function getMinimalCommitmentDiscountManager(
        Service_OpenApi_ApiInterface $openApi,
        Service_PbaApi_ApiInterface $pbaApi
    ) {
        return new Model_MinimalCommitmentDiscountManager($openApi, $pbaApi);
    }
}
