<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Model_OSA_SubscriptionParametersManager implements Model_OSA_SubscriptionParametersManagerInterface
{
   const LOG_EVENT_CANNOT_GET_PROVISIONING_PARAMETER = 'Cannot get provisioning parameter "%s". Error: "%s"';

    /** @var Service_PbaApi_ApiInterface */
    private $_pbaApi;

    /**
     * @param Service_PbaApi_ApiInterface $pbaApi
     */
    public function __construct(Service_PbaApi_ApiInterface $pbaApi)
    {
        $this->_pbaApi = $pbaApi;
    }

    /**
     * @param int $subscriptionId
     * @return int
     * @throws Exception
     */
    public function getParameterDatacenterId($subscriptionId)
    {
        return (int) $this->_pbaApi->getSubscriptionParameterValue($subscriptionId, self::PARAMETER_DATACENTER);
    }

    /**
     * @param int $subscriptionId
     * @return string
     * @throws Exception
     */
    public function getParameterStorageId($subscriptionId)
    {
        return $this->_pbaApi->getSubscriptionParameterValue($subscriptionId, self::PARAMETER_STORAGE);
    }

    /**
     * @param int $subscriptionId
     * @return bool
     * @throws Exception
     */
    public function getParameterIsExternal($subscriptionId)
    {
        try {
            return (bool) $this->_pbaApi->getSubscriptionParameterValue($subscriptionId, self::PARAMETER_IS_EXTERNAL_GROUP);
        } catch (Service_PbaApi_Api_Exception $e) {
            Log::getInstance()->event(sprintf(
                self::LOG_EVENT_CANNOT_GET_PROVISIONING_PARAMETER,
                self::PARAMETER_IS_EXTERNAL_GROUP,
                $e->getMessage()
            ));

            if ($e->isSubscriptionParameterNotExistsException()) {
                return null;
            }
            throw $e;
        }
    }

    /**
     * @param int $subscriptionId
     * @return int
     * @throws Exception
     */
    public function getParameterGroupId($subscriptionId)
    {
        try {
            return (int) $this->_pbaApi->getSubscriptionParameterValue($subscriptionId, self::PARAMETER_GROUP_ID);
        } catch (Service_PbaApi_Api_Exception $e) {
            Log::getInstance()->event(sprintf(
                self::LOG_EVENT_CANNOT_GET_PROVISIONING_PARAMETER,
                self::PARAMETER_GROUP_ID,
                $e->getMessage()
            ));

            if ($e->isSubscriptionParameterNotExistsException()) {
                return null;
            }
            throw $e;
        }
    }

    /**
     * @param int $subscriptionId
     * @return bool
     * @throws Exception
     */
    public function getParameterWSDiscount($subscriptionId)
    {
        try {
            return (boolean) $this->_pbaApi->getSubscriptionParameterValue($subscriptionId, self::PARAMETER_WS_DISCOUNT);
        } catch (Service_PbaApi_Api_Exception $e) {
            Log::getInstance()->event(sprintf(
                self::LOG_EVENT_CANNOT_GET_PROVISIONING_PARAMETER,
                self::PARAMETER_WS_DISCOUNT,
                $e->getMessage()
            ));

            if ($e->isSubscriptionParameterNotExistsException()) {
                return false;
            }
            throw $e;
        }
    }

    /**
     * @param $subscriptionId
     * @param int $value
     */
    public function setParameterExcludedFromReport($subscriptionId, $value)
    {
        $this->_pbaApi->updateSubscriptionParameterValue(
            $subscriptionId,
            self::PARAMETER_EXCLUDE_FROM_REPORT,
            $value
        );
    }

    /**
     * @param $subscriptionId
     * @return int
     * @throws Service_PbaApi_Api_Exception
     */
    public function getParameterExcludedFromReport($subscriptionId)
    {
        try {
            return (int) $this->_pbaApi->getSubscriptionParameterValue($subscriptionId, self::PARAMETER_EXCLUDE_FROM_REPORT);
        } catch (Service_PbaApi_Api_Exception $e) {
            Log::getInstance()->event(sprintf(
                self::LOG_EVENT_CANNOT_GET_PROVISIONING_PARAMETER,
                self::PARAMETER_EXCLUDE_FROM_REPORT,
                $e->getMessage()
            ));

            if ($e->isSubscriptionParameterNotExistsException()) {
                return 0;
            }
            throw $e;
        }
    }

    /**
     * @param $subscriptionId
     * @return int
     * @throws Service_PbaApi_Api_Exception
     */
    public function getParameterDiscount($subscriptionId)
    {
        try {
            return (int) $this->_pbaApi->getSubscriptionParameterValue($subscriptionId, self::PARAMETER_DISCOUNT);
        } catch (Service_PbaApi_Api_Exception $e) {
            Log::getInstance()->event(sprintf(
                self::LOG_EVENT_CANNOT_GET_PROVISIONING_PARAMETER,
                self::PARAMETER_DISCOUNT,
                $e->getMessage()
            ));

            if ($e->isSubscriptionParameterNotExistsException()) {
                return 0;
            }
            throw $e;
        }
    }

    public function getParameterPassword($subscriptionId)
    {
        try {
            return $this->_pbaApi->getSubscriptionParameterValue($subscriptionId, self::PARAMETER_PASSWORD);
        } catch (Service_PbaApi_Api_Exception $e) {
            Log::getInstance()->event(sprintf(
                self::LOG_EVENT_CANNOT_GET_PROVISIONING_PARAMETER,
                self::PARAMETER_PASSWORD,
                $e->getMessage()
            ));

            if ($e->isSubscriptionParameterNotExistsException()) {
                return null;
            }
            throw $e;
        }
    }

    /**
     * @param $subscriptionId
     * @param int $value
     */
    public function setParameterPassword($subscriptionId, $value)
    {
        $this->_pbaApi->updateSubscriptionParameterValue(
            $subscriptionId,
            self::PARAMETER_PASSWORD,
            $value
        );
    }
}
