<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

class Model_ResellerMapByDatacenterHelper implements Model_ResellerMapHelperInterface
{
    /**
     * @var \APS\ControllerProxy
     */
    private $_apsc;
    /**
     * @var \APS\ResourceProxy | \APS\ResourceBase
     */
    private $_global;
    /**
     * @var string
     */
    private $_datacenterId;

    private $resellerRelationsWereChanged = false;


    /**
     * Model_ResellerMapHelper constructor.
     * @param \APS\ControllerProxy $apsc
     * @param \APS\ResourceProxy | \APS\ResourceBase $global
     * @throws Exception
     */
    public function __construct($apsc, $global, $datacenter)
    {
        $this->_apsc = $apsc->impersonate($global);
        $this->_global = $this->_apsc->getResource($global->aps->id);
        $this->_datacenterId = $datacenter->id;
    }

    /**
     * @param int $accountId
     * @return int | null
     */
    public function getResellerIdFromMap($accountId)
    {
        if (!is_array($this->_global->resellerRelations))
        {
            return null;
        }

        foreach ($this->_global->resellerRelations as $reseller)
        {
            if ($reseller->datacenterId == $this->_datacenterId && $reseller->accountId == $accountId)
            {
                return $reseller->groupId;
            }
        }
        return null;
    }

    /**
     * @param int $accountId
     * @param int $groupId
     */
    public function addResellerIdToMap($accountId, $groupId)
    {
        if (!is_array($this->_global->resellerRelations))
        {
            $this->_global->resellerRelations = array();
        }

        $reseller = new ResellerRelation();
        $reseller->datacenterId = $this->_datacenterId;
        $reseller->accountId = $accountId;
        $reseller->groupId = $groupId;
        $this->_global->resellerRelations[] = $reseller;
        $this->resellerRelationsWereChanged = true;
    }

    public function updateResellerMap()
    {
        if ($this->resellerRelationsWereChanged) {
            $this->_apsc->updateResource($this->_global);
        }
    }
}