<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\Services\Cache;

use Acronis\App\Services\Config\CacheProvidersEnum;
use Acronis\App\Services\Config\ConfigAwareTrait;
use Acronis\App\Services\Config\ConfigSchema;
use Acronis\App\Services\Cache\Exception\UnsupportedProvider;
use Acronis\App\Services\FactoryInterface;
use Acronis\App\Services\Redis\RedisAwareTrait;
use Arr;
use Str;

class CacheFactory implements FactoryInterface
{
    const NAME = 'cache';

    use ConfigAwareTrait;
    use RedisAwareTrait;

    /**
     * @throws UnsupportedProvider
     * @return CacheInterface
     */
    public function createInstance()
    {
        $cacheConfig = $this->getCacheConfig();
        $cacheProviderName = $cacheConfig[ConfigSchema::NODE_CACHE_PROVIDER];
        $prefix = Arr::get($cacheConfig, ConfigSchema::NODE_CACHE_PREFIX, '');
        $ttl = Arr::get($cacheConfig, ConfigSchema::NODE_CACHE_TTL);

        return $this->createCacheProvider($cacheProviderName, $prefix, $ttl);
    }

    private function getCacheConfig()
    {
        return $this->getConfig()->getCacheConfig();
    }

    /**
     * @param $cacheProviderName
     * @throws UnsupportedProvider
     * @return RedisCache
     */
    private function createCacheProvider($cacheProviderName, $keyPrefix, $ttl)
    {
        if ($cacheProviderName === CacheProvidersEnum::CACHE_PROVIDER_REDIS) {
            return new RedisCache($this->getRedis(), $keyPrefix, $ttl);
        }

        throw new UnsupportedProvider(Str::format(
            'Unknown cache provider "%s".',
            $cacheProviderName
        ));
    }
}