<?php
/**
 * @Copyright © 2002-2018 Acronis International GmbH. All rights reserved
 */

namespace Acronis\App\UsageReport;

use Acronis\App\Utils\CsvFileIterator;

class CsvReportIterator extends CsvFileIterator
{
    private $firstRowOffset;

    private $formatParser;

    /**
     * ReportRowIterator constructor.
     * @param string $filePath
     * @param FormatParserInterface $formatParser
     * @throws Exception\RuntimeException
     * @throws \RuntimeException
     */
    public function __construct($filePath, $formatParser)
    {
        $this->formatParser = $formatParser;
        $fileHandler = gzopen($filePath, 'r');
        parent::__construct($fileHandler);

        $columnNames = $this->readNext();
        $this->formatParser->setColumnNames($columnNames);
        $this->firstRowOffset = ftell($this->fileHandler);
    }

    public function rewind()
    {
        parent::rewind();
        fseek($this->fileHandler, $this->firstRowOffset);
    }

    /**
     * @return ReportRowWrapper|null
     * @throws Exception\RuntimeException
     * @throws \Exception
     */
    public function current()
    {
        $row = parent::current();

        return $this->formatParser->parseRow($row);
    }
}