<?php
/**
 * @Copyright © 2002-2020 Acronis International GmbH. All rights reserved
 */

use Acronis\App\Services\Config\ConfigFactory;
use Acronis\App\Services\Config\ConfigSchema;
use Acronis\App\Services\Locator;
use Acronis\App\Services\Logger\ApsLoggerAdapter;

if (!ini_get('date.timezone')) {
    $timezone = exec('date +%Z');
    date_default_timezone_set(timezone_name_from_abbr($timezone));
}

require_once 'config.php';
require_once DIR_LIB . 'autoload.php';

if (!DEVELOPMENT_MODE) {
    require_once('aps/2/runtime.php');
}

if (getenv('BUILD_NUMBER') === false && !DEVELOPMENT_MODE) {
    Errors::setHandlers();
}
$logger = Log::createLogger();
Log::getInstance()->setLogger($logger);
\APS\LoggerRegistry::set(new ApsLoggerAdapter());

if (!DEVELOPMENT_MODE) {
    $locator = Locator::getInstance();
    $services = require 'services.php';
    foreach ($services as $serviceName => $serviceFactory) {
        $locator->addFactory($serviceName, $serviceFactory);
    }
}

if (getenv('BUILD_NUMBER') === false && !DEVELOPMENT_MODE) {
    $config = Locator::getInstance()->get(ConfigFactory::NAME);
    $logConfig = $config->getLogConfig();
    if ($logConfig[ConfigSchema::NODE_LOG_USE_ADDITIONAL]) {
        $additionalLogger = Log::createLogger(Log::ADDITIONAL_LOGGER_NAME);
        Log::getInstance()->setAdditionalLogger($additionalLogger);
    }
}

