<?php
/**
 * @Copyright © 2002-2017 Acronis International GmbH. All rights reserved
 */

/**
 * It's a tool to test generation of correcting orders
 * The script sets up property "accountToProcess" for the resource "globals"
 * Place the script into /var/www/html/backupservice
 *
 * Usage: php addAccountToProcessingList.php <APS INSTANCE UUID> <ACCOUNT ID> <TYPE> <DATE> '<SUBSCRIPTION ID 1[, SUBSCRIPTION ID 2, ...]>'
 *
 * Example: php addAccountToProcessingList.php e36529b7-dca5-4201-9517-483618be6e53 1006819 onboarding 13.10.2016 '1000001, 1000002, 1000003, 1000004'
 */

require_once(__DIR__ . "/../app/bootstrap.php");

chdir(DIR_ROOT);
define('APS_RUNTIME_HOME', DIR_ROOT);
define('APS_RUNTIME_NO_PROCESS', true);

$_SERVER['HTTP_APS_INSTANCE_ID'] = $argv[1];

$accountId = intval($argv[2]);
$type = $argv[3];
$date = strtotime($argv[4]);
$subscriptions = array_map(function ($id) {
    return intval(trim($id));
}, explode(',', $argv[5]));

if (!in_array($type, array('onboarding', 'cancellation'))) {
    echo "Type must be onboarding or cancellation";
    exit(1);
}

$url = 'https://' . OPEN_API_URI . ':6308/';

echo "Initial APS controller...\n";
$apsc = new \APS\ControllerProxy($url);
APS\Request::setController($apsc);
$resources = $apsc->getResources("implementing(" . \APS\TypeLibrary::getTypeByServiceId('globals') . ")");
$globals = reset($resources);

$record = new AccountToProcess();
$record->accountId = $accountId;
$record->date = $date;
$record->type = $type;
$record->subscriptions = $subscriptions;

$globals->accountToProcess = $globals->accountToProcess ?: array();
$globals->accountToProcess[] = $record;

$res = $apsc->updateResource($globals);

print_r($res);
echo "Account was successfully added to processing list\n";
